#ifndef DATAPARSER_H
#define DATAPARSER_H

#include "FileData.h"

#include <QObject>
#include <Datas.h>
#include <QFile>
#include <QXmlStreamWriter>


class DataParser : public QObject
{

    Q_OBJECT

public:
    explicit DataParser(QObject *parent = 0);
    ~DataParser();
    ServerSettings* GetServerSettings();
    void CreateServerSettings(QString server,QString port);
    void CreateXML(QList<FileData> fileDataList);
    void CreateAuthMessage(ClientAutorization *auth);
    void CreateAuthData(ServerAuthorization *serverAuth);

private:
    ClientAutorization *authPassCache;

};

#endif // DATAPARSER_H
